package com.ekexiu.project.platform.views.service;

import com.ekexiu.project.platform.views.dao.DayDao;
import com.ekexiu.project.platform.views.handler.LogHandler;
import com.ekexiu.project.platform.views.pojo.ArticleSum;
import com.ekexiu.project.platform.views.pojo.DayData;
import org.jfw.apt.annotation.Autowrie;
import org.jfw.apt.annotation.DefaultValue;
import org.jfw.apt.annotation.Nullable;
import org.jfw.apt.web.annotation.Path;
import org.jfw.apt.web.annotation.operate.Get;
import org.jfw.apt.web.annotation.param.JdbcConn;
import org.jfw.apt.web.annotation.param.ParameterMap;
import org.jfw.util.PageQueryResult;
import org.jfw.util.jdbc.JdbcUtil;
import org.jfw.util.jdbc.PreparedStatementConfig;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.List;
import java.util.Map;

@Path("/log")
public class Main {
    public static final String MAX_DAY = "99999999";
    public static final String MIN_DAY = "00000000";


    private List<LogHandler> handlers;

    @Autowrie
    private DayDao dayDao;

    public List<LogHandler> getHandlers() {
        return handlers;
    }

    public void setHandlers(List<LogHandler> handlers) {
        this.handlers = handlers;
    }

    public DayDao getDayDao() {
        return dayDao;
    }

    public void setDayDao(DayDao dayDao) {
        this.dayDao = dayDao;
    }

    @Get
    @Path()
    public void log(@ParameterMap Map<String, String[]> pm) {
        for (LogHandler lh : this.handlers) {
            lh.add(pm);
        }
    }

    @Get
    @Path("/qm")
    public List<DayData> query(@JdbcConn Connection con, String tn, @Nullable String[] src, @Nullable String[] id, @DefaultValue("com.ekexiu.project.platform.views.service.Main.MIN_DAY") String bt,
                               @DefaultValue("com.ekexiu.project.platform.views.service.Main.MAX_DAY") String et, @Nullable String sort) throws SQLException {
        return dayDao.query(con, tn, src, id, bt, et, sort);
    }

    @Get
    @Path("/qo")
    public List<DayData> query(@JdbcConn Connection con, String tn, String id, @DefaultValue("com.ekexiu.project.platform.views.service.Main.MIN_DAY") String bt,
                               @DefaultValue("com.ekexiu.project.platform.views.service.Main.MAX_DAY") String et) throws SQLException {
        return dayDao.query(con, tn, id, bt, et);
    }

    @Get
    @Path("/qs")
    public List<DayData> query(@JdbcConn Connection con, String tn, @Nullable String[] src, @DefaultValue("com.ekexiu.project.platform.views.service.Main.MIN_DAY") String bt,
                               @DefaultValue("com.ekexiu.project.platform.views.service.Main.MAX_DAY") String et, @DefaultValue("20") int rows) throws SQLException {
        return dayDao.query(con, tn, bt, et, src, rows);
    }

    @Get
    @Path("/article/pq")
    public PageQueryResult<ArticleSum> query(@JdbcConn Connection con, @Nullable String catalog, @Nullable String title, @Nullable String source, @Nullable String creator, @Nullable Boolean published, @Nullable String bt, @Nullable String et, String orderBy, int pageSize, int pageNo) throws SQLException {
        if (orderBy.equals("createTime")) {
            return dayDao.queryByTime(con, catalog, title == null ? null : "%" + title + "%", source == null ? null : "%" + source + "%", creator, published, bt, et, pageSize, pageNo);
        } else if (orderBy.equals("views")) {
            return dayDao.queryByViews(con, catalog, title == null ? null : "%" + title + "%", source == null ? null : "%" + source + "%", creator, published, bt, et, pageSize, pageNo);
        }
        return null;
    }

    @Get
    @Path("/article/total")
    public List<Map<String, Object>> total(@JdbcConn Connection con, @DefaultValue("com.ekexiu.project.platform.views.service.Main.MIN_DAY") final String bt) throws SQLException {
        return JdbcUtil.queryMaps(con, "SELECT day,sum(pv)as pv,source FROM article_day WHERE day>=? GROUP BY day,source ORDER BY day DESC ", new PreparedStatementConfig() {
            @Override
            public void config(PreparedStatement preparedStatement) throws SQLException {
                preparedStatement.setString(1, bt);
            }
        });
    }
}
